@@ -34,6 +34,7 @@
 #include "pager.h"
 #include "charset.h"
 #include "mutt_crypt.h"
+#include "mutt_random.h"
 #include "mutt_idna.h"
 #include "buffy.h"
 #include "send.h"
@@ -79,8 +80,6 @@ const char B64Chars[64] = {
   '8', '9', '+', '/'
 };
 
-static char MsgIdPfx = 'A';
-
 static void transform_to_7bit (BODY *a, FILE *fpin);
 
 static void encode_quoted (FGETCONV * fc, FILE *fout, int istext)
@@ -427,7 +426,8 @@ int mutt_write_mime_header (BODY *a, FILE *f)
         ) &&
       a->mime_headers)
   {
-    mutt_write_rfc822_header (f, a->mime_headers, NULL, MUTT_WRITE_HEADER_MIME, 0, 0);
+    mutt_write_rfc822_header (f, a->mime_headers, NULL, a->mime_headers->date,
+                              MUTT_WRITE_HEADER_MIME, 0, 0);
   }
 
   /* Do NOT add the terminator here!!! */
@@ -515,13 +515,8 @@ int mutt_write_mime_body (BODY *a, FILE *f)
 void mutt_generate_boundary (PARAMETER **parm)
 {
   char rs[BOUNDARYLEN + 1];
-  char *p = rs;
-  int i;
 
-  rs[BOUNDARYLEN] = 0;
-  for (i=0;i<BOUNDARYLEN;i++)
-    *p++ = B64Chars[LRAND() % sizeof (B64Chars)];
-  *p = 0;
+  mutt_base64_random96(rs);
 
   mutt_set_parameter ("boundary", rs, parm);
 }
@@ -1717,7 +1712,10 @@ BODY *mutt_remove_multipart_alternative (BODY *b)
   return b;
 }
 
-char *mutt_make_date (char *s, size_t len)
+/* Appends the date to the passed in buffer.
+ * The buffer is not cleared because some callers prepend quotes.
+ */
+void mutt_make_date (BUFFER *s)
 {
   time_t t = time (NULL);
   struct tm *l = localtime (&t);
@@ -1725,11 +1723,10 @@ char *mutt_make_date (char *s, size_t len)
 
   tz /= 60;
 
-  snprintf (s, len,  "Date: %s, %d %s %d %02d:%02d:%02d %+03d%02d\n",
+  mutt_buffer_add_printf (s,  "%s, %d %s %d %02d:%02d:%02d %+03d%02d",
 	    Weekdays[l->tm_wday], l->tm_mday, Months[l->tm_mon],
 	    l->tm_year + 1900, l->tm_hour, l->tm_min, l->tm_sec,
 	    (int) tz / 60, (int) abs ((int) tz) % 60);
-  return (s);
 }
 
 /* wrapper around mutt_write_address() so we can handle very large
@@ -2165,6 +2162,10 @@ out:
  *
  * Likewise, all IDN processing should happen outside of this routine.
  *
+ * date can be NULL, otherwise it should be the output of
+ * mutt_make_date().  This is passed in explicitly to prevent
+ * accidental date setting from "garbage" in env->date.
+ *
  * mode == MUTT_WRITE_HEADER_EDITHDRS  => "lite" mode (used for edit_hdrs)
  * mode == MUTT_WRITE_HEADER_NORMAL    => normal mode.  write full header + MIME headers
  * mode == MUTT_WRITE_HEADER_FCC       => fcc mode, like normal mode but for Bcc header
@@ -2179,7 +2180,7 @@ out:
  * $crypt_protected_headers_subject in NORMAL or POSTPONE mode.
  *
  */
-int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
+int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach, char *date,
 			      mutt_write_header_mode mode, int privacy,
                               int hide_protected_subject)
 {
@@ -2188,9 +2189,20 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
   LIST *tmp = env->userhdrs;
   int has_agent = 0; /* user defined user-agent header field exists */
 
-  if ((mode == MUTT_WRITE_HEADER_NORMAL || mode == MUTT_WRITE_HEADER_FCC) &&
+  if ((mode == MUTT_WRITE_HEADER_NORMAL || mode == MUTT_WRITE_HEADER_FCC ||
+       mode == MUTT_WRITE_HEADER_POSTPONE || mode == MUTT_WRITE_HEADER_MIME) &&
       !privacy)
-    fputs (mutt_make_date (buffer, sizeof(buffer)), fp);
+  {
+    if (date)
+      fprintf (fp, "Date: %s\n", date);
+    else
+    {
+      BUFFER *datebuf = mutt_buffer_pool_get ();
+      mutt_make_date (datebuf);
+      fprintf (fp, "Date: %s\n", mutt_b2s (datebuf));
+      mutt_buffer_pool_release (&datebuf);
+    }
+  }
 
   /* OPTUSEFROM is not consulted here so that we can still write a From:
    * field if the user sets it with the `my_hdr' command
@@ -2401,19 +2413,28 @@ const char *mutt_fqdn(short may_hide_host)
 char *mutt_gen_msgid (void)
 {
   char buf[SHORT_STRING];
-  time_t now;
-  struct tm *tm;
+  time_t now = time (NULL);
+  char random_bytes[8];
+  char localpart[12]; /* = 32 bit timestamp, plus 64 bit randomness */
+  unsigned char localpart_B64[16+1]; /* = Base64 encoded value of localpart plus
+                                        terminating \0 */
   const char *fqdn;
 
-  now = time (NULL);
-  tm = gmtime (&now);
-  if (!(fqdn = mutt_fqdn(0)))
-    fqdn = NONULL(Hostname);
+  mutt_random_bytes (random_bytes, sizeof(random_bytes));
 
-  snprintf (buf, sizeof (buf), "<%d%02d%02d%02d%02d%02d.G%c%u@%s>",
-	    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour,
-	    tm->tm_min, tm->tm_sec, MsgIdPfx, (unsigned int)getpid (), fqdn);
-  MsgIdPfx = (MsgIdPfx == 'Z') ? 'A' : MsgIdPfx + 1;
+  /* Convert the four least significant bytes of our timestamp and put it in
+     localpart, with proper endianness (for humans) taken into account. */
+  for (int i = 0; i < 4; i++)
+    localpart[i] = (uint8_t) (now >> (3-i)*8u);
+
+  memcpy (&localpart[4], &random_bytes, 8);
+
+  mutt_to_base64 (localpart_B64, (unsigned char *) localpart, 12, 17);
+
+  if (!(fqdn = mutt_fqdn (0)))
+    fqdn = NONULL (Hostname);
+
+  snprintf (buf, sizeof (buf), "<%s@%s>", localpart_B64, fqdn);
   return (safe_strdup (buf));
 }
 
@@ -2811,7 +2832,6 @@ static int _mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to, const char *r
 {
   int i, ret = 0;
   FILE *f;
-  char date[SHORT_STRING];
   BUFFER *tempfile;
   MESSAGE *msg = NULL;
 
@@ -2836,13 +2856,19 @@ static int _mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to, const char *r
   {
     int ch_flags = CH_XMIT | CH_NONEWLINE | CH_NOQFROM;
     char* msgid_str;
+    BUFFER *date;
 
     if (!option (OPTBOUNCEDELIVERED))
       ch_flags |= CH_WEED_DELIVERED;
 
     fseeko (fp, h->offset, 0);
-    fprintf (f, "Resent-From: %s", resent_from);
-    fprintf (f, "\nResent-%s", mutt_make_date (date, sizeof(date)));
+    fprintf (f, "Resent-From: %s\n", resent_from);
+
+    date = mutt_buffer_pool_get ();
+    mutt_make_date (date);
+    fprintf (f, "Resent-Date: %s\n", mutt_b2s (date));
+    mutt_buffer_pool_release (&date);
+
     msgid_str = mutt_gen_msgid();
     fprintf (f, "Resent-Message-ID: %s\n", msgid_str);
     fputs ("Resent-To: ", f);
@@ -2990,7 +3016,6 @@ int mutt_write_fcc (const char *path, SEND_CONTEXT *sctx, const char *msgid, int
   FILE *tempfp = NULL;
   int r = -1, need_buffy_cleanup = 0;
   struct stat st;
-  char buf[SHORT_STRING];
   int onm_flags;
 
   hdr = sctx->msg;
@@ -3036,7 +3061,7 @@ int mutt_write_fcc (const char *path, SEND_CONTEXT *sctx, const char *msgid, int
   /* post == 1 => postpone message.
    * post == 0 => fcc mode.
    * */
-  mutt_write_rfc822_header (msg->fp, hdr->env, hdr->content,
+  mutt_write_rfc822_header (msg->fp, hdr->env, hdr->content, sctx->date_header,
                             post ? MUTT_WRITE_HEADER_POSTPONE : MUTT_WRITE_HEADER_FCC,
                             0,
                             option (OPTCRYPTPROTHDRSREAD) &&
@@ -3060,11 +3085,6 @@ int mutt_write_fcc (const char *path, SEND_CONTEXT *sctx, const char *msgid, int
   if (f.magic == MUTT_MMDF || f.magic == MUTT_MBOX)
     fprintf (msg->fp, "Status: RO\n");
 
-  /* mutt_write_rfc822_header() only writes out a Date: header with
-   * mode == 0, i.e. _not_ postponment; so write out one ourself */
-  if (post)
-    fprintf (msg->fp, "%s", mutt_make_date (buf, sizeof (buf)));
-
   /* (postponment) if the mail is to be signed or encrypted, save this info */
   if ((WithCrypto & APPLICATION_PGP)
       && post && (hdr->security & APPLICATION_PGP))
